; MIT License

; Copyright (c) 2020 María Belén Mera Reyes, Pablo Ramón Guevara

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

;;
;; ENTITY MANAGER
;;

.include "man_entity.h.s"
.include "cpctelera.h.s"
.include "sys/sys_collision.h.s"
.include "sys/sys_render.h.s"



_player_initial_data: DefineEntity 40, 0, 5, 19, 0, 0, 1, _sp_personaje
_player:: .ds entity_size
_obstacle_array:: ;;Array para los obstáculos
	.ds max_obstacles*entity_size

_obstacle_array_end:: .ds 0

_next_obstacle_ptr:: .dw _obstacle_array  ;; ALMACENO UN PUNTERO A LA POSICIÓN FINAL DEL ARRAY PARA PODER AÑADIR OTRO ELEMENTO

_man_entity_init::  ;; crea un jugador y limpia los obstáculos
	ld hl, #_obstacle_array
	ld (_next_obstacle_ptr), hl

	ld hl, #_player_initial_data
	ld bc, #entity_size
	ld de, #_player
	ldir

	ret

;; crea una entidad desde hl en _next_obstacle_ptr, excepto si no caben más
;;Entrada
;; HL: puntero a los bytes de inicialización
_man_entity_create_obstacle::
	ld bc, #_obstacle_array_end
	ld de, (_next_obstacle_ptr)
	ld a, b
	cp d
	jp nz, #copiar
	ld a, c
	cp e
	ret z ; reached end of array

	copiar:
	ld de, (_next_obstacle_ptr)	
	ld bc, #entity_size
	ldir

	; incrementar _next_obstacle_ptr
	ld hl, (_next_obstacle_ptr)
	ld bc, #entity_size
	add hl, bc
	ld (_next_obstacle_ptr), hl

	ret

;; cambia la entidad en hl por la última entidad
;; limpia la pantalla
;; decrementa _last_elem_ptr y ix
;; Entradas:
;; ix: entity to be destroyed

;; B is 1 if obstacles is empty, 0 otherwise
_man_entity_obstacles_empty::
	ld b, #0
	ld hl, #_obstacle_array
	ld de, (_next_obstacle_ptr)
	ld a, h
	cp d
	ret nz
	ld a, l
	cp e
	ret nz
	ld b, #1
	ret

_man_entity_destroy_obstacle::

	; limpiar
	call _clear_previous_entity_position

	; copiar entidad en _next_obstacle_ptr - 1 a ix
	; de = entity to be destroyed
	ld__d_ixh
	ld__e_ixl

	ld hl, (_next_obstacle_ptr)
	ld bc, #-entity_size
	add hl, bc
	ld bc, #entity_size
	ldir

	; ; actualizar puntero a última entidad
	ld bc, #-entity_size
	add ix, bc ; decrementar ix
	ld hl, (_next_obstacle_ptr)
	add hl, bc
	ld (_next_obstacle_ptr), hl

	ret

;; Entradas
;; HL: función que recibe ix como entrada
_man_entity_run_on_player::
	ld ix, #_player
	jp (hl)

;; run function on hl on every entity
;; uses ix for keeping track of current obstacle,
;; so the function on hl has to keep that in mind
;; Entradas
;; HL: función que recibe ix como entrada
_man_entity_run_on_obstacles::
	ld ix, #_obstacle_array
	loop:
		ld de, (_next_obstacle_ptr)
		ld a, d
		cp__ixh
		jp nz, #inside
		ld a, e
		cp__ixl
		ret z ; reached end of array
	inside:
		push hl ; almacena la direccion de la llamada para que no se pierda
		
		; indirect call
		ld (llamada), hl
		llamada = .+1
		call #0x0000

		pop hl
		
		; incrementar ix
		ld bc, #entity_size
		add ix, bc
		
		jp loop
