; MIT License

; Copyright (c) 2020 María Belén Mera Reyes, Pablo Ramón Guevara

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

;; SISTEMA DE COLISIONES
.include "sys_collision.h.s"
.include "man/man_entity.h.s"
.include "sys/sys_collision.h.s"
.include "cpctelera.h.s"
.include "man/man_menu.h.s"
.include "man/man_game.h.s"
.include "sys/sys_physics.h.s"

    
_sys_collision_update::
    ld iy, #_player
    jp _collides_update

_collides_update::
    ld hl, #_collides
    jp _man_entity_run_on_obstacles

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_collides::
    ; o_x < p_x + p_w
    ld a, e_x(iy)
    add a, e_w(iy)
    ld b, a
    ld a, e_x(ix)
    cp b
    ret nc
    ; o_y < p_y + p_h
    ld a, e_y(iy)
    add a, e_h(iy)
    ld b, a
    ld a, e_y(ix)
    cp b
    ret nc
    ; p_x < o_x + o_w
    ld a, e_x(ix)
    add a, e_w(ix)
    ld b, a
    ld a, e_x(iy)
    cp b
    ret nc
    ; p_y < o_y + o_h
    ld a, e_y(ix)
    add a, e_h(ix)
    ld b, a
    ld a, e_y(iy)
    cp b
    ret nc
    ; collides
    _top_right::
        ; p_x < o_x
        ld b, e_x(ix)
        ld a, e_x(iy)
        cp b
        jp nc, _top_left
        ; p_y < o_y
        ld b, e_y(ix)
        ld a, e_y(iy)
        cp b
        jp nc, _bottom_right
        ; collide
        jp _collision_top_right
        ret
    _bottom_right::
        ; o_y + o_h < p_y + p_h
        ld a, e_y(iy)
        add a, e_h(iy)
        ld b, a
        ld a, e_y(ix)
        add a, e_h(ix)
        cp b
        jp nc, _middle_right
        ; collide
        jp _collision_bottom_right
        ret
    _middle_right::
        jp _push_left
    _top_left::
        ; o_x + o_w < p_x + p_w
        ld a, e_x(iy)
        add a, e_w(iy)
        ld b, a
        ld a, e_x(ix)
        add a, e_w(ix)
        cp b
        jp nc, _middle_top
        ; p_y < o_y
        ld b, e_y(ix)
        ld a, e_y(iy)
        cp b
        jp nc, _bottom_left
        ; collide
        jp _collision_top_left
        ret
    _bottom_left::
        ; o_y + o_h < p_y + p_h
        ld a, e_y(iy)
        add a, e_h(iy)
        ld b, a
        ld a, e_y(ix)
        add a, e_h(ix)
        cp b
        jp nc, _middle_left
        ; collide
        jp _collision_bottom_left
        ret
    _middle_left::
        ; p_x < o_x + o_w
        ld a, e_x(ix)
        add a, e_w(ix)
        ld b, a
        ld a, e_x(iy)
        cp b
        jp nc, _middle_top
        jp _push_right
    _middle_top::
        ; p_y < o_y
        ld b, e_y(ix)
        ld a, e_y(iy)
        cp b
        jp nc, _middle_bottom
        jp _push_down
    _middle_bottom::
        ; push up
        jp _push_up
        ret

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_collision_bottom_right::
    ; i = p_w - o_x + p_x
    ld a, e_w(iy)
    sub a, e_x(ix)
    add a, e_x(iy)
    ld b, a
    ; j = o_y + o_h - p_y
    ld a, e_y(ix)
    add a, e_h(ix)
    sub a, e_y(iy)
    ; if i < j push_left else push_up
    cp b
    jp c, _brpu
        jp _push_left
    _brpu:
        jp _push_up

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_collision_top_right::
    ; i = p_w - o_x + p_x
    ld a, e_w(iy)
    sub a, e_x(ix)
    add a, e_x(iy)
    ld b, a
    ; j = p_h - o_y + p_y
    ld a, e_h(iy)
    sub a, e_y(ix)
    add a, e_y(iy)
    ; if i < j push_left else push_down
    cp b
    jp c, _trpd
        jp _push_left
    _trpd:
        jp _push_down

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_collision_bottom_left::
    ; i = o_w - p_x + o_x
    ld a, e_w(ix)
    sub a, e_x(iy)
    add a, e_x(ix)
    ld b, a
    ; j = o_h + o_y - p_y
    ld a, e_h(ix)
    add a, e_y(ix)
    sub a, e_y(iy)
    ; if i < j push_right else push_up
    cp b
    jp c, _blpu
        jp _push_right
    _blpu:
        jp _push_up

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_collision_top_left::
    ; i = p_w - p_x + o_x
    ld a, e_w(iy)
    sub a, e_x(iy)
    add a, e_x(ix)
    ld b, a
    ; j = p_h + o_y - p_y
    ld a, e_h(iy)
    add a, e_y(ix)
    sub a, e_y(iy)
    ; if i < j push_left else push_down
    cp b
    jp c, _tlpd
        jp _push_right
    _tlpd:
        jp _push_down

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_push_right::
    ld a, e_x(ix)
    add a, e_w(ix)
    ld e_x(iy), a
    ret

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_push_left::
    ld a, e_x(ix)
    sub a, e_w(iy)
    ld e_x(iy), a
    ret

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_push_up::
    ld a, e_y(ix)
    add a, e_h(ix)
    ld e_y(iy), a
    xor a
    ld e_vy(iy), a
    ld hl, #_touching_ground
    ld (hl), #1
    ret

;; ENTRADAS:
;;ix: obstaculo
;;iy: jugador
_push_down::
    ld a, e_y(ix)
    sub a, e_h(iy)
    ld e_y(iy), a
    xor a
    ld e_vy(iy), a
    ret