; MIT License

; Copyright (c) 2020 María Belén Mera Reyes, Pablo Ramón Guevara

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

;;INPUT SYSTEM
.include "cpctelera.h.s"
.include "keyboard/keyboard.h.s"
.include "sys_input.h.s"
.include "sys_physics.h.s"
.include "man/man_entity.h.s"

_sys_input_init::
    ret

_sys_input_update::
    call cpct_scanKeyboard_f_asm ;; escaneo el teclado

    ld hl, #_variable_jump_control_update
    call _man_entity_run_on_player

    ld hl, #_side_movement_update
    call _man_entity_run_on_player

    ret

_side_movement_update::
    ld hl, #Key_O

    call cpct_isKeyPressed_asm
    jr z, O_notPressed
    O_pressed:
        ld e_vx(ix), #-2
        ret
    O_notPressed:
        ld hl, #Key_P  ;; la P indica movimiento a la derecha
        call cpct_isKeyPressed_asm
        jr z, P_notPressed

        P_pressed:
            ld e_vx(ix), #1
            ret
        P_notPressed:
            ld e_vx(ix), #0
    ret

   


;; Controla el estado del salto con la barra espaciadora y actualiza
;; la gravedad y la velocidad del jugador
;; requiere haber llamado a scanKeyboard antes
;; Entradas
;; ix: player addr
_variable_jump_control_update::
    ld a, (_variable_jump_state)
    cp #0
    jp z, ground
    cp #1
    jp z, rising
    falling:
        ld a, (_touching_ground)
        or a
        ret z

        trans_ground:
            ld hl, #_variable_jump_state
            ld (hl), #ground_state
            ld a, (_gravity)
            sub #extra_gravity
            ld (_gravity), a
            ret
        
    ground:
        ld hl, #Key_Space
        call cpct_isKeyPressed_asm
        ret z ; stay in ground if spacebar not pressed
        ; trans_rising
            ld hl, #_touching_ground
            ld (hl), #0
            ld e_vy(ix), #jump_initial_speed
            ld hl, #_variable_jump_state
            ld (hl), #rising_state
            ret

    rising:
        ; stay while vy > 0 && spacebar pressed
        ld hl, #Key_Space
        call cpct_isKeyPressed_asm
        jp z, trans_falling ; spacebar released
        ; spacebar pressed
            ld a, e_vy(ix)
            cp #0
            ret p ; vy is positive
        trans_falling:
            ld hl, #_variable_jump_state
            ld (hl), #falling_state
            ; set gravity
            ld a, (_gravity)
            add a, #extra_gravity
            ld (_gravity), a
            ret