; MIT License

; Copyright (c) 2020 María Belén Mera Reyes, Pablo Ramón Guevara

; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.

;;
;; PHYSICS SYSTEM
;;
.include "cpctelera.h.s"
.include "sys_physics.h.s"
.include "man/man_entity.h.s"
.include "man/man_menu.h.s"
.include "man/man_game.h.s"
.include "sys_render.h.s"
.include "man/man_level.h.s"
.include "man/man_world.h.s"

; .include "sys/sys_collision.h.s"

_physics_init::
    ret

_gravity:: .db #-1

_touching_ground:: .db #1

_variable_jump_state:: .db 1

;; ENTRADAS
;; ix: player addr
_sys_player_jump_physics_update::

    ; first update y' = y + vy
    ld a, e_vy(ix)
    sra a ; divide by 2
    add a, e_y(ix) ; a = y + vy

    cp #0 ; cp 0
    jp z, #below

    cp #200-19
    jp nc, below
        ; y > ground_level
        ld e_y(ix), a ; store y'

        ; then compute vy'
        ld a, (_gravity)
        add a, e_vy(ix)

        ; max terminal velocity
        cp #terminal_velocity
        jp p, store ; vy < terminal
            ld a, #terminal_velocity
        store:
            ld e_vy(ix), a ; store vy'
        ret
    below:
        ld e_y(ix), #ground_level
        ld e_vy(ix), #0
        ld hl, #_touching_ground
        ld (hl), #1
        ret

_sys_scroll::
    dec e_x(ix)
    ret

;; ENTRADAS
;; ix: player
_sys_right_bound_player::
    ld a, e_x(ix)
    cp #80-15
    ret c
    ld e_x(ix), #80-15
    ret

_sys_update_lateral_position::
    ld a, e_vx(ix)
    add a, e_x(ix)
    ld e_x(ix), a
    ret

_sys_physics_delete_left_obstacles::
    ld a, e_x(ix)
    cp #80
    ret c

    call _man_entity_destroy_obstacle
    ret

;; ENTRADA
;;  ix: puntero al player
_sys_physics_update::
    ld hl, #_sys_scroll
    call _man_entity_run_on_obstacles

    ld hl, #_sys_player_jump_physics_update
    call _man_entity_run_on_player

    ld hl, #_sys_update_lateral_position
    call _man_entity_run_on_player
    call _man_entity_run_on_obstacles

    ld hl, #_sys_right_bound_player
    call _man_entity_run_on_player

    ld hl, #_sys_physics_delete_left_obstacles
    call _man_entity_run_on_obstacles

    ret